import numpy as np
import re

# supported types 

simple_types = [
#   name, aliases, columns_class name
    ("int32", ["int16","i4","<i4"], "Int32Array"),
    ("int64", ["<i8","i8"], "Int64Array"),
    ("float32", ["<f4","f4"], "Float32Array"),
    ("float64", ["<f8","f8"], "Float64Array"),
    ("bool", ["boolean"], "BoolArray"),
    ("string", ["str","O","o","object"],  "StringArray"),
]

simple_array_types = [

    ("int32array", ["<i4","i4"], "Int32ArrayArray"),
    ("int64array", ["<i8","i8"], "Int64ArrayArray"),
    ("float32array", ["<f4","f4"], "Float32ArrayArray"),
    ("float64array", ["<f8","f8"], "Float64ArrayArray"),
]

class TypeTester:
    def test(self, name):
        raise NotImplementedError

    def cast(self,x):
        return x

class SimpleTester(TypeTester):
    def __init__(self, name, aliases, column_class):
        self.name = name
        self.aliases = aliases
        self.column_class = column_class
    
    def test(self,name):
        if (name.lower() == self.name) or (name in self.aliases):
            return True
        return False

    def cast(self,x):
        if self.name=='string':
            return x.astype('str')
        return x

class SimpleArrayTester(TypeTester):
    def __init__(self, name, aliases, column_class):
        self.name = name
        self.aliases = aliases
        self.column_class = column_class
        self.ms = [re.compile("\('<{}', \(\d*,\)\)".format(alias)) for alias in aliases]

    def test(self, name):
        for m in self.ms:
            res = m.match(name)
            if res:
                return True
        return False

type_testers = [SimpleTester(*p) for p in simple_types] + [SimpleArrayTester(*p) for p in simple_array_types]



from __future__ import print_function
import grpc
from . import straxrpc_pb2
from . import straxrpc_pb2_grpc
import pandas as pd
import numpy as np


class StraxClient:
    def __init__(self, addr="localhost:50051"):
        self.addr = addr

    def search_field(self, pattern):
         with grpc.insecure_channel(self.addr) as channel:
            stub = straxrpc_pb2_grpc.StraxRPCStub(channel)
            sp = straxrpc_pb2.SearchPattern(pattern=pattern)
            rs = stub.SearchField(sp)
            results = [f"{r.name} is part of {r.data_name} (provided by {r.plugin})" for r in rs]
            # print(results)
            return results

    def data_info(self,dataname):
        with grpc.insecure_channel(self.addr) as channel:
            stub = straxrpc_pb2_grpc.StraxRPCStub(channel)
            ti = straxrpc_pb2.TableInfo(name=dataname)
        
            data = {}
            for col in stub.DataInfo(ti):
                vs = getattr(col, col.info.dtype).values
                data[col.info.name] = pd.Series(index=col.index, data=vs)
            df = pd.DataFrame(data)
            return df

    def get_df(self, run_id, dframe):
       with grpc.insecure_channel(self.addr) as channel:
            stub = straxrpc_pb2_grpc.StraxRPCStub(channel)
            ti = straxrpc_pb2.TableInfo(name=dframe, run_id=run_id)
            data = {}
            for col in stub.GetDataframe(ti):
                vs = getattr(col, col.info.dtype).values
                data[col.info.name] = pd.Series(index=col.index, data=vs)
            df = pd.DataFrame(data)
            return df

    def get_array(self, run_id, dframe):
        with grpc.insecure_channel(self.addr) as channel:
            stub = straxrpc_pb2_grpc.StraxRPCStub(channel)
            ti = straxrpc_pb2.TableInfo(name=dframe, run_id=run_id)
            # f = straxrpc_pb2.ColumnInfo(name='random')
            data = []
            names = []
            formats = []

            for col in stub.GetArray(ti):
                vs = getattr(col, col.info.dtype).values
                data.append(vs)
                names.append(col.info.name)
                formats.append(col.info.dtype)
            
            dtype = dict(names = names, formats=formats)
            arr = np.fromiter(zip(*data), dtype=dtype)
            return arr

    def search_dataframe_names(self, pattern):
        with grpc.insecure_channel(self.addr) as channel:
            stub = straxrpc_pb2_grpc.StraxRPCStub(channel)
            sp = straxrpc_pb2.SearchPattern(pattern=pattern,)
            rs = stub.SearchDataframeNames(sp)
            names = [x.name for x in rs]
            # print(names)
            return names

    def show_config(self, name):
        with grpc.insecure_channel(self.addr) as channel:
            stub = straxrpc_pb2_grpc.StraxRPCStub(channel)
            ti = straxrpc_pb2.TableInfo(name=name)
            data = {}
            for col in stub.ShowConfig(ti):
                vs = getattr(col, col.info.dtype).values
                data[col.info.name] = pd.Series(index=col.index, data=vs)
            df = pd.DataFrame(data)
            return df

def run():
    client = StraxClient()
    print("---------------------- Search for s1* ----------------------")
    res = client.search_field( "s1*")
    print("\n".join(res))
    print("\n-------------- Data Info for event_basics --------------\n")
    df = client.data_info( 'event_basics')
    print(df)
    print("\n--------- Get event_basics df for 180423_1021 ---------\n")
    df = client.get_df( '180423_1021', 'event_basics')
    print(df)
    print("\n--------- Get event_basics array for 180423_1021 ---------\n")
    arr = client.get_array( '180423_1021', 'event_basics')
    print(arr.dtype.names)
    print(arr)

if __name__ == '__main__':
    run()



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: straxrpc/straxrpc.proto

import sys
_b=sys.version_info[0]<3 and (lambda x:x) or (lambda x:x.encode('latin1'))
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
from google.protobuf import descriptor_pb2
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor.FileDescriptor(
  name='straxrpc/straxrpc.proto',
  package='straxrpc',
  syntax='proto3',
  serialized_pb=_b('\n\x17straxrpc/straxrpc.proto\x12\x08straxrpc\"0\n\x06\x43onfig\x12\x13\n\x0bpax_raw_dir\x18\x01 \x01(\t\x12\x11\n\tinput_dir\x18\x02 \x01(\t\" \n\rSearchPattern\x12\x0f\n\x07pattern\x18\x01 \x01(\t\"P\n\tTableInfo\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x0e\n\x06run_id\x18\x02 \x01(\t\x12%\n\x07\x63olumns\x18\x03 \x03(\x0b\x32\x14.straxrpc.ColumnInfo\"U\n\nPluginInfo\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x12\n\nclass_name\x18\x02 \x01(\t\x12%\n\x07\x63olumns\x18\x03 \x03(\x0b\x32\x14.straxrpc.ColumnInfo\"]\n\nColumnInfo\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x11\n\tdata_name\x18\x02 \x01(\t\x12\x0e\n\x06plugin\x18\x03 \x01(\t\x12\x0f\n\x07\x63omment\x18\x04 \x01(\t\x12\r\n\x05\x64type\x18\x05 \x01(\t\"\x1c\n\nInt32Array\x12\x0e\n\x06values\x18\x01 \x03(\x05\"\x1c\n\nInt64Array\x12\x0e\n\x06values\x18\x01 \x03(\x03\"7\n\x0fInt32ArrayArray\x12$\n\x06values\x18\x01 \x03(\x0b\x32\x14.straxrpc.Int32Array\"7\n\x0fInt64ArrayArray\x12$\n\x06values\x18\x01 \x03(\x0b\x32\x14.straxrpc.Int64Array\"\x1b\n\tBoolArray\x12\x0e\n\x06values\x18\x01 \x03(\x08\"\x1e\n\x0c\x46loat32Array\x12\x0e\n\x06values\x18\x01 \x03(\x02\"\x1e\n\x0c\x46loat64Array\x12\x0e\n\x06values\x18\x01 \x03(\x01\";\n\x11\x46loat32ArrayArray\x12&\n\x06values\x18\x01 \x03(\x0b\x32\x16.straxrpc.Float32Array\";\n\x11\x46loat64ArrayArray\x12&\n\x06values\x18\x01 \x03(\x0b\x32\x16.straxrpc.Float64Array\"\x1d\n\x0bStringArray\x12\x0e\n\x06values\x18\x01 \x03(\t\"\xae\x04\n\nDataColumn\x12\"\n\x04info\x18\x63 \x01(\x0b\x32\x14.straxrpc.ColumnInfo\x12\r\n\x05index\x18\x01 \x03(\r\x12%\n\x05int32\x18\x02 \x01(\x0b\x32\x14.straxrpc.Int32ArrayH\x00\x12%\n\x05int64\x18\x03 \x01(\x0b\x32\x14.straxrpc.Int64ArrayH\x00\x12)\n\x07\x66loat32\x18\x04 \x01(\x0b\x32\x16.straxrpc.Float32ArrayH\x00\x12)\n\x07\x66loat64\x18\x05 \x01(\x0b\x32\x16.straxrpc.Float64ArrayH\x00\x12\'\n\x06string\x18\x06 \x01(\x0b\x32\x15.straxrpc.StringArrayH\x00\x12#\n\x04\x62ool\x18\x07 \x01(\x0b\x32\x13.straxrpc.BoolArrayH\x00\x12\'\n\x06object\x18\x08 \x01(\x0b\x32\x15.straxrpc.StringArrayH\x00\x12\x33\n\x0c\x66loat32array\x18\t \x01(\x0b\x32\x1b.straxrpc.Float32ArrayArrayH\x00\x12\x33\n\x0c\x66loat64array\x18\n \x01(\x0b\x32\x1b.straxrpc.Float64ArrayArrayH\x00\x12/\n\nint32array\x18\x0b \x01(\x0b\x32\x19.straxrpc.Int32ArrayArrayH\x00\x12/\n\nint64array\x18\x0c \x01(\x0b\x32\x19.straxrpc.Int64ArrayArrayH\x00\x42\x06\n\x04\x64\x61ta\"Y\n\x0bStraxConfig\x12$\n\x05names\x18\x01 \x01(\x0b\x32\x15.straxrpc.StringArray\x12$\n\x06values\x18\x02 \x01(\x0b\x32\x14.straxrpc.DataColumn2\x89\x03\n\x08StraxRPC\x12@\n\x0bSearchField\x12\x17.straxrpc.SearchPattern\x1a\x14.straxrpc.ColumnInfo\"\x00\x30\x01\x12I\n\x14SearchDataframeNames\x12\x17.straxrpc.SearchPattern\x1a\x14.straxrpc.PluginInfo\"\x00\x30\x01\x12\x39\n\x08\x44\x61taInfo\x12\x13.straxrpc.TableInfo\x1a\x14.straxrpc.DataColumn\"\x00\x30\x01\x12=\n\x0cGetDataframe\x12\x13.straxrpc.TableInfo\x1a\x14.straxrpc.DataColumn\"\x00\x30\x01\x12\x39\n\x08GetArray\x12\x13.straxrpc.TableInfo\x1a\x14.straxrpc.DataColumn\"\x00\x30\x01\x12;\n\nShowConfig\x12\x13.straxrpc.TableInfo\x1a\x14.straxrpc.DataColumn\"\x00\x30\x01\x62\x06proto3')
)




_CONFIG = _descriptor.Descriptor(
  name='Config',
  full_name='straxrpc.Config',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='pax_raw_dir', full_name='straxrpc.Config.pax_raw_dir', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='input_dir', full_name='straxrpc.Config.input_dir', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=37,
  serialized_end=85,
)


_SEARCHPATTERN = _descriptor.Descriptor(
  name='SearchPattern',
  full_name='straxrpc.SearchPattern',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='pattern', full_name='straxrpc.SearchPattern.pattern', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=87,
  serialized_end=119,
)


_TABLEINFO = _descriptor.Descriptor(
  name='TableInfo',
  full_name='straxrpc.TableInfo',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='name', full_name='straxrpc.TableInfo.name', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='run_id', full_name='straxrpc.TableInfo.run_id', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='columns', full_name='straxrpc.TableInfo.columns', index=2,
      number=3, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=121,
  serialized_end=201,
)


_PLUGININFO = _descriptor.Descriptor(
  name='PluginInfo',
  full_name='straxrpc.PluginInfo',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='name', full_name='straxrpc.PluginInfo.name', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='class_name', full_name='straxrpc.PluginInfo.class_name', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='columns', full_name='straxrpc.PluginInfo.columns', index=2,
      number=3, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=203,
  serialized_end=288,
)


_COLUMNINFO = _descriptor.Descriptor(
  name='ColumnInfo',
  full_name='straxrpc.ColumnInfo',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='name', full_name='straxrpc.ColumnInfo.name', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='data_name', full_name='straxrpc.ColumnInfo.data_name', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='plugin', full_name='straxrpc.ColumnInfo.plugin', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='comment', full_name='straxrpc.ColumnInfo.comment', index=3,
      number=4, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='dtype', full_name='straxrpc.ColumnInfo.dtype', index=4,
      number=5, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=290,
  serialized_end=383,
)


_INT32ARRAY = _descriptor.Descriptor(
  name='Int32Array',
  full_name='straxrpc.Int32Array',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='values', full_name='straxrpc.Int32Array.values', index=0,
      number=1, type=5, cpp_type=1, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=385,
  serialized_end=413,
)


_INT64ARRAY = _descriptor.Descriptor(
  name='Int64Array',
  full_name='straxrpc.Int64Array',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='values', full_name='straxrpc.Int64Array.values', index=0,
      number=1, type=3, cpp_type=2, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=415,
  serialized_end=443,
)


_INT32ARRAYARRAY = _descriptor.Descriptor(
  name='Int32ArrayArray',
  full_name='straxrpc.Int32ArrayArray',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='values', full_name='straxrpc.Int32ArrayArray.values', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=445,
  serialized_end=500,
)


_INT64ARRAYARRAY = _descriptor.Descriptor(
  name='Int64ArrayArray',
  full_name='straxrpc.Int64ArrayArray',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='values', full_name='straxrpc.Int64ArrayArray.values', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=502,
  serialized_end=557,
)


_BOOLARRAY = _descriptor.Descriptor(
  name='BoolArray',
  full_name='straxrpc.BoolArray',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='values', full_name='straxrpc.BoolArray.values', index=0,
      number=1, type=8, cpp_type=7, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=559,
  serialized_end=586,
)


_FLOAT32ARRAY = _descriptor.Descriptor(
  name='Float32Array',
  full_name='straxrpc.Float32Array',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='values', full_name='straxrpc.Float32Array.values', index=0,
      number=1, type=2, cpp_type=6, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=588,
  serialized_end=618,
)


_FLOAT64ARRAY = _descriptor.Descriptor(
  name='Float64Array',
  full_name='straxrpc.Float64Array',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='values', full_name='straxrpc.Float64Array.values', index=0,
      number=1, type=1, cpp_type=5, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=620,
  serialized_end=650,
)


_FLOAT32ARRAYARRAY = _descriptor.Descriptor(
  name='Float32ArrayArray',
  full_name='straxrpc.Float32ArrayArray',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='values', full_name='straxrpc.Float32ArrayArray.values', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=652,
  serialized_end=711,
)


_FLOAT64ARRAYARRAY = _descriptor.Descriptor(
  name='Float64ArrayArray',
  full_name='straxrpc.Float64ArrayArray',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='values', full_name='straxrpc.Float64ArrayArray.values', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=713,
  serialized_end=772,
)


_STRINGARRAY = _descriptor.Descriptor(
  name='StringArray',
  full_name='straxrpc.StringArray',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='values', full_name='straxrpc.StringArray.values', index=0,
      number=1, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=774,
  serialized_end=803,
)


_DATACOLUMN = _descriptor.Descriptor(
  name='DataColumn',
  full_name='straxrpc.DataColumn',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='info', full_name='straxrpc.DataColumn.info', index=0,
      number=99, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='index', full_name='straxrpc.DataColumn.index', index=1,
      number=1, type=13, cpp_type=3, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='int32', full_name='straxrpc.DataColumn.int32', index=2,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='int64', full_name='straxrpc.DataColumn.int64', index=3,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='float32', full_name='straxrpc.DataColumn.float32', index=4,
      number=4, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='float64', full_name='straxrpc.DataColumn.float64', index=5,
      number=5, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='string', full_name='straxrpc.DataColumn.string', index=6,
      number=6, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='bool', full_name='straxrpc.DataColumn.bool', index=7,
      number=7, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='object', full_name='straxrpc.DataColumn.object', index=8,
      number=8, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='float32array', full_name='straxrpc.DataColumn.float32array', index=9,
      number=9, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='float64array', full_name='straxrpc.DataColumn.float64array', index=10,
      number=10, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='int32array', full_name='straxrpc.DataColumn.int32array', index=11,
      number=11, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='int64array', full_name='straxrpc.DataColumn.int64array', index=12,
      number=12, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
    _descriptor.OneofDescriptor(
      name='data', full_name='straxrpc.DataColumn.data',
      index=0, containing_type=None, fields=[]),
  ],
  serialized_start=806,
  serialized_end=1364,
)


_STRAXCONFIG = _descriptor.Descriptor(
  name='StraxConfig',
  full_name='straxrpc.StraxConfig',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='names', full_name='straxrpc.StraxConfig.names', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='values', full_name='straxrpc.StraxConfig.values', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1366,
  serialized_end=1455,
)

_TABLEINFO.fields_by_name['columns'].message_type = _COLUMNINFO
_PLUGININFO.fields_by_name['columns'].message_type = _COLUMNINFO
_INT32ARRAYARRAY.fields_by_name['values'].message_type = _INT32ARRAY
_INT64ARRAYARRAY.fields_by_name['values'].message_type = _INT64ARRAY
_FLOAT32ARRAYARRAY.fields_by_name['values'].message_type = _FLOAT32ARRAY
_FLOAT64ARRAYARRAY.fields_by_name['values'].message_type = _FLOAT64ARRAY
_DATACOLUMN.fields_by_name['info'].message_type = _COLUMNINFO
_DATACOLUMN.fields_by_name['int32'].message_type = _INT32ARRAY
_DATACOLUMN.fields_by_name['int64'].message_type = _INT64ARRAY
_DATACOLUMN.fields_by_name['float32'].message_type = _FLOAT32ARRAY
_DATACOLUMN.fields_by_name['float64'].message_type = _FLOAT64ARRAY
_DATACOLUMN.fields_by_name['string'].message_type = _STRINGARRAY
_DATACOLUMN.fields_by_name['bool'].message_type = _BOOLARRAY
_DATACOLUMN.fields_by_name['object'].message_type = _STRINGARRAY
_DATACOLUMN.fields_by_name['float32array'].message_type = _FLOAT32ARRAYARRAY
_DATACOLUMN.fields_by_name['float64array'].message_type = _FLOAT64ARRAYARRAY
_DATACOLUMN.fields_by_name['int32array'].message_type = _INT32ARRAYARRAY
_DATACOLUMN.fields_by_name['int64array'].message_type = _INT64ARRAYARRAY
_DATACOLUMN.oneofs_by_name['data'].fields.append(
  _DATACOLUMN.fields_by_name['int32'])
_DATACOLUMN.fields_by_name['int32'].containing_oneof = _DATACOLUMN.oneofs_by_name['data']
_DATACOLUMN.oneofs_by_name['data'].fields.append(
  _DATACOLUMN.fields_by_name['int64'])
_DATACOLUMN.fields_by_name['int64'].containing_oneof = _DATACOLUMN.oneofs_by_name['data']
_DATACOLUMN.oneofs_by_name['data'].fields.append(
  _DATACOLUMN.fields_by_name['float32'])
_DATACOLUMN.fields_by_name['float32'].containing_oneof = _DATACOLUMN.oneofs_by_name['data']
_DATACOLUMN.oneofs_by_name['data'].fields.append(
  _DATACOLUMN.fields_by_name['float64'])
_DATACOLUMN.fields_by_name['float64'].containing_oneof = _DATACOLUMN.oneofs_by_name['data']
_DATACOLUMN.oneofs_by_name['data'].fields.append(
  _DATACOLUMN.fields_by_name['string'])
_DATACOLUMN.fields_by_name['string'].containing_oneof = _DATACOLUMN.oneofs_by_name['data']
_DATACOLUMN.oneofs_by_name['data'].fields.append(
  _DATACOLUMN.fields_by_name['bool'])
_DATACOLUMN.fields_by_name['bool'].containing_oneof = _DATACOLUMN.oneofs_by_name['data']
_DATACOLUMN.oneofs_by_name['data'].fields.append(
  _DATACOLUMN.fields_by_name['object'])
_DATACOLUMN.fields_by_name['object'].containing_oneof = _DATACOLUMN.oneofs_by_name['data']
_DATACOLUMN.oneofs_by_name['data'].fields.append(
  _DATACOLUMN.fields_by_name['float32array'])
_DATACOLUMN.fields_by_name['float32array'].containing_oneof = _DATACOLUMN.oneofs_by_name['data']
_DATACOLUMN.oneofs_by_name['data'].fields.append(
  _DATACOLUMN.fields_by_name['float64array'])
_DATACOLUMN.fields_by_name['float64array'].containing_oneof = _DATACOLUMN.oneofs_by_name['data']
_DATACOLUMN.oneofs_by_name['data'].fields.append(
  _DATACOLUMN.fields_by_name['int32array'])
_DATACOLUMN.fields_by_name['int32array'].containing_oneof = _DATACOLUMN.oneofs_by_name['data']
_DATACOLUMN.oneofs_by_name['data'].fields.append(
  _DATACOLUMN.fields_by_name['int64array'])
_DATACOLUMN.fields_by_name['int64array'].containing_oneof = _DATACOLUMN.oneofs_by_name['data']
_STRAXCONFIG.fields_by_name['names'].message_type = _STRINGARRAY
_STRAXCONFIG.fields_by_name['values'].message_type = _DATACOLUMN
DESCRIPTOR.message_types_by_name['Config'] = _CONFIG
DESCRIPTOR.message_types_by_name['SearchPattern'] = _SEARCHPATTERN
DESCRIPTOR.message_types_by_name['TableInfo'] = _TABLEINFO
DESCRIPTOR.message_types_by_name['PluginInfo'] = _PLUGININFO
DESCRIPTOR.message_types_by_name['ColumnInfo'] = _COLUMNINFO
DESCRIPTOR.message_types_by_name['Int32Array'] = _INT32ARRAY
DESCRIPTOR.message_types_by_name['Int64Array'] = _INT64ARRAY
DESCRIPTOR.message_types_by_name['Int32ArrayArray'] = _INT32ARRAYARRAY
DESCRIPTOR.message_types_by_name['Int64ArrayArray'] = _INT64ARRAYARRAY
DESCRIPTOR.message_types_by_name['BoolArray'] = _BOOLARRAY
DESCRIPTOR.message_types_by_name['Float32Array'] = _FLOAT32ARRAY
DESCRIPTOR.message_types_by_name['Float64Array'] = _FLOAT64ARRAY
DESCRIPTOR.message_types_by_name['Float32ArrayArray'] = _FLOAT32ARRAYARRAY
DESCRIPTOR.message_types_by_name['Float64ArrayArray'] = _FLOAT64ARRAYARRAY
DESCRIPTOR.message_types_by_name['StringArray'] = _STRINGARRAY
DESCRIPTOR.message_types_by_name['DataColumn'] = _DATACOLUMN
DESCRIPTOR.message_types_by_name['StraxConfig'] = _STRAXCONFIG
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

Config = _reflection.GeneratedProtocolMessageType('Config', (_message.Message,), dict(
  DESCRIPTOR = _CONFIG,
  __module__ = 'straxrpc.straxrpc_pb2'
  # @@protoc_insertion_point(class_scope:straxrpc.Config)
  ))
_sym_db.RegisterMessage(Config)

SearchPattern = _reflection.GeneratedProtocolMessageType('SearchPattern', (_message.Message,), dict(
  DESCRIPTOR = _SEARCHPATTERN,
  __module__ = 'straxrpc.straxrpc_pb2'
  # @@protoc_insertion_point(class_scope:straxrpc.SearchPattern)
  ))
_sym_db.RegisterMessage(SearchPattern)

TableInfo = _reflection.GeneratedProtocolMessageType('TableInfo', (_message.Message,), dict(
  DESCRIPTOR = _TABLEINFO,
  __module__ = 'straxrpc.straxrpc_pb2'
  # @@protoc_insertion_point(class_scope:straxrpc.TableInfo)
  ))
_sym_db.RegisterMessage(TableInfo)

PluginInfo = _reflection.GeneratedProtocolMessageType('PluginInfo', (_message.Message,), dict(
  DESCRIPTOR = _PLUGININFO,
  __module__ = 'straxrpc.straxrpc_pb2'
  # @@protoc_insertion_point(class_scope:straxrpc.PluginInfo)
  ))
_sym_db.RegisterMessage(PluginInfo)

ColumnInfo = _reflection.GeneratedProtocolMessageType('ColumnInfo', (_message.Message,), dict(
  DESCRIPTOR = _COLUMNINFO,
  __module__ = 'straxrpc.straxrpc_pb2'
  # @@protoc_insertion_point(class_scope:straxrpc.ColumnInfo)
  ))
_sym_db.RegisterMessage(ColumnInfo)

Int32Array = _reflection.GeneratedProtocolMessageType('Int32Array', (_message.Message,), dict(
  DESCRIPTOR = _INT32ARRAY,
  __module__ = 'straxrpc.straxrpc_pb2'
  # @@protoc_insertion_point(class_scope:straxrpc.Int32Array)
  ))
_sym_db.RegisterMessage(Int32Array)

Int64Array = _reflection.GeneratedProtocolMessageType('Int64Array', (_message.Message,), dict(
  DESCRIPTOR = _INT64ARRAY,
  __module__ = 'straxrpc.straxrpc_pb2'
  # @@protoc_insertion_point(class_scope:straxrpc.Int64Array)
  ))
_sym_db.RegisterMessage(Int64Array)

Int32ArrayArray = _reflection.GeneratedProtocolMessageType('Int32ArrayArray', (_message.Message,), dict(
  DESCRIPTOR = _INT32ARRAYARRAY,
  __module__ = 'straxrpc.straxrpc_pb2'
  # @@protoc_insertion_point(class_scope:straxrpc.Int32ArrayArray)
  ))
_sym_db.RegisterMessage(Int32ArrayArray)

Int64ArrayArray = _reflection.GeneratedProtocolMessageType('Int64ArrayArray', (_message.Message,), dict(
  DESCRIPTOR = _INT64ARRAYARRAY,
  __module__ = 'straxrpc.straxrpc_pb2'
  # @@protoc_insertion_point(class_scope:straxrpc.Int64ArrayArray)
  ))
_sym_db.RegisterMessage(Int64ArrayArray)

BoolArray = _reflection.GeneratedProtocolMessageType('BoolArray', (_message.Message,), dict(
  DESCRIPTOR = _BOOLARRAY,
  __module__ = 'straxrpc.straxrpc_pb2'
  # @@protoc_insertion_point(class_scope:straxrpc.BoolArray)
  ))
_sym_db.RegisterMessage(BoolArray)

Float32Array = _reflection.GeneratedProtocolMessageType('Float32Array', (_message.Message,), dict(
  DESCRIPTOR = _FLOAT32ARRAY,
  __module__ = 'straxrpc.straxrpc_pb2'
  # @@protoc_insertion_point(class_scope:straxrpc.Float32Array)
  ))
_sym_db.RegisterMessage(Float32Array)

Float64Array = _reflection.GeneratedProtocolMessageType('Float64Array', (_message.Message,), dict(
  DESCRIPTOR = _FLOAT64ARRAY,
  __module__ = 'straxrpc.straxrpc_pb2'
  # @@protoc_insertion_point(class_scope:straxrpc.Float64Array)
  ))
_sym_db.RegisterMessage(Float64Array)

Float32ArrayArray = _reflection.GeneratedProtocolMessageType('Float32ArrayArray', (_message.Message,), dict(
  DESCRIPTOR = _FLOAT32ARRAYARRAY,
  __module__ = 'straxrpc.straxrpc_pb2'
  # @@protoc_insertion_point(class_scope:straxrpc.Float32ArrayArray)
  ))
_sym_db.RegisterMessage(Float32ArrayArray)

Float64ArrayArray = _reflection.GeneratedProtocolMessageType('Float64ArrayArray', (_message.Message,), dict(
  DESCRIPTOR = _FLOAT64ARRAYARRAY,
  __module__ = 'straxrpc.straxrpc_pb2'
  # @@protoc_insertion_point(class_scope:straxrpc.Float64ArrayArray)
  ))
_sym_db.RegisterMessage(Float64ArrayArray)

StringArray = _reflection.GeneratedProtocolMessageType('StringArray', (_message.Message,), dict(
  DESCRIPTOR = _STRINGARRAY,
  __module__ = 'straxrpc.straxrpc_pb2'
  # @@protoc_insertion_point(class_scope:straxrpc.StringArray)
  ))
_sym_db.RegisterMessage(StringArray)

DataColumn = _reflection.GeneratedProtocolMessageType('DataColumn', (_message.Message,), dict(
  DESCRIPTOR = _DATACOLUMN,
  __module__ = 'straxrpc.straxrpc_pb2'
  # @@protoc_insertion_point(class_scope:straxrpc.DataColumn)
  ))
_sym_db.RegisterMessage(DataColumn)

StraxConfig = _reflection.GeneratedProtocolMessageType('StraxConfig', (_message.Message,), dict(
  DESCRIPTOR = _STRAXCONFIG,
  __module__ = 'straxrpc.straxrpc_pb2'
  # @@protoc_insertion_point(class_scope:straxrpc.StraxConfig)
  ))
_sym_db.RegisterMessage(StraxConfig)



_STRAXRPC = _descriptor.ServiceDescriptor(
  name='StraxRPC',
  full_name='straxrpc.StraxRPC',
  file=DESCRIPTOR,
  index=0,
  options=None,
  serialized_start=1458,
  serialized_end=1851,
  methods=[
  _descriptor.MethodDescriptor(
    name='SearchField',
    full_name='straxrpc.StraxRPC.SearchField',
    index=0,
    containing_service=None,
    input_type=_SEARCHPATTERN,
    output_type=_COLUMNINFO,
    options=None,
  ),
  _descriptor.MethodDescriptor(
    name='SearchDataframeNames',
    full_name='straxrpc.StraxRPC.SearchDataframeNames',
    index=1,
    containing_service=None,
    input_type=_SEARCHPATTERN,
    output_type=_PLUGININFO,
    options=None,
  ),
  _descriptor.MethodDescriptor(
    name='DataInfo',
    full_name='straxrpc.StraxRPC.DataInfo',
    index=2,
    containing_service=None,
    input_type=_TABLEINFO,
    output_type=_DATACOLUMN,
    options=None,
  ),
  _descriptor.MethodDescriptor(
    name='GetDataframe',
    full_name='straxrpc.StraxRPC.GetDataframe',
    index=3,
    containing_service=None,
    input_type=_TABLEINFO,
    output_type=_DATACOLUMN,
    options=None,
  ),
  _descriptor.MethodDescriptor(
    name='GetArray',
    full_name='straxrpc.StraxRPC.GetArray',
    index=4,
    containing_service=None,
    input_type=_TABLEINFO,
    output_type=_DATACOLUMN,
    options=None,
  ),
  _descriptor.MethodDescriptor(
    name='ShowConfig',
    full_name='straxrpc.StraxRPC.ShowConfig',
    index=5,
    containing_service=None,
    input_type=_TABLEINFO,
    output_type=_DATACOLUMN,
    options=None,
  ),
])
_sym_db.RegisterServiceDescriptor(_STRAXRPC)

DESCRIPTOR.services_by_name['StraxRPC'] = _STRAXRPC

# @@protoc_insertion_point(module_scope)



# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
import grpc

from straxrpc import straxrpc_pb2 as straxrpc_dot_straxrpc__pb2


class StraxRPCStub(object):
  # missing associated documentation comment in .proto file
  pass

  def __init__(self, channel):
    """Constructor.

    Args:
      channel: A grpc.Channel.
    """
    self.SearchField = channel.unary_stream(
        '/straxrpc.StraxRPC/SearchField',
        request_serializer=straxrpc_dot_straxrpc__pb2.SearchPattern.SerializeToString,
        response_deserializer=straxrpc_dot_straxrpc__pb2.ColumnInfo.FromString,
        )
    self.SearchDataframeNames = channel.unary_stream(
        '/straxrpc.StraxRPC/SearchDataframeNames',
        request_serializer=straxrpc_dot_straxrpc__pb2.SearchPattern.SerializeToString,
        response_deserializer=straxrpc_dot_straxrpc__pb2.PluginInfo.FromString,
        )
    self.DataInfo = channel.unary_stream(
        '/straxrpc.StraxRPC/DataInfo',
        request_serializer=straxrpc_dot_straxrpc__pb2.TableInfo.SerializeToString,
        response_deserializer=straxrpc_dot_straxrpc__pb2.DataColumn.FromString,
        )
    self.GetDataframe = channel.unary_stream(
        '/straxrpc.StraxRPC/GetDataframe',
        request_serializer=straxrpc_dot_straxrpc__pb2.TableInfo.SerializeToString,
        response_deserializer=straxrpc_dot_straxrpc__pb2.DataColumn.FromString,
        )
    self.GetArray = channel.unary_stream(
        '/straxrpc.StraxRPC/GetArray',
        request_serializer=straxrpc_dot_straxrpc__pb2.TableInfo.SerializeToString,
        response_deserializer=straxrpc_dot_straxrpc__pb2.DataColumn.FromString,
        )
    self.ShowConfig = channel.unary_stream(
        '/straxrpc.StraxRPC/ShowConfig',
        request_serializer=straxrpc_dot_straxrpc__pb2.TableInfo.SerializeToString,
        response_deserializer=straxrpc_dot_straxrpc__pb2.DataColumn.FromString,
        )


class StraxRPCServicer(object):
  # missing associated documentation comment in .proto file
  pass

  def SearchField(self, request, context):
    # missing associated documentation comment in .proto file
    pass
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def SearchDataframeNames(self, request, context):
    # missing associated documentation comment in .proto file
    pass
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def DataInfo(self, request, context):
    # missing associated documentation comment in .proto file
    pass
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def GetDataframe(self, request, context):
    # missing associated documentation comment in .proto file
    pass
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def GetArray(self, request, context):
    # missing associated documentation comment in .proto file
    pass
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def ShowConfig(self, request, context):
    # missing associated documentation comment in .proto file
    pass
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')


def add_StraxRPCServicer_to_server(servicer, server):
  rpc_method_handlers = {
      'SearchField': grpc.unary_stream_rpc_method_handler(
          servicer.SearchField,
          request_deserializer=straxrpc_dot_straxrpc__pb2.SearchPattern.FromString,
          response_serializer=straxrpc_dot_straxrpc__pb2.ColumnInfo.SerializeToString,
      ),
      'SearchDataframeNames': grpc.unary_stream_rpc_method_handler(
          servicer.SearchDataframeNames,
          request_deserializer=straxrpc_dot_straxrpc__pb2.SearchPattern.FromString,
          response_serializer=straxrpc_dot_straxrpc__pb2.PluginInfo.SerializeToString,
      ),
      'DataInfo': grpc.unary_stream_rpc_method_handler(
          servicer.DataInfo,
          request_deserializer=straxrpc_dot_straxrpc__pb2.TableInfo.FromString,
          response_serializer=straxrpc_dot_straxrpc__pb2.DataColumn.SerializeToString,
      ),
      'GetDataframe': grpc.unary_stream_rpc_method_handler(
          servicer.GetDataframe,
          request_deserializer=straxrpc_dot_straxrpc__pb2.TableInfo.FromString,
          response_serializer=straxrpc_dot_straxrpc__pb2.DataColumn.SerializeToString,
      ),
      'GetArray': grpc.unary_stream_rpc_method_handler(
          servicer.GetArray,
          request_deserializer=straxrpc_dot_straxrpc__pb2.TableInfo.FromString,
          response_serializer=straxrpc_dot_straxrpc__pb2.DataColumn.SerializeToString,
      ),
      'ShowConfig': grpc.unary_stream_rpc_method_handler(
          servicer.ShowConfig,
          request_deserializer=straxrpc_dot_straxrpc__pb2.TableInfo.FromString,
          response_serializer=straxrpc_dot_straxrpc__pb2.DataColumn.SerializeToString,
      ),
  }
  generic_handler = grpc.method_handlers_generic_handler(
      'straxrpc.StraxRPC', rpc_method_handlers)
  server.add_generic_rpc_handlers((generic_handler,))



"""The Python implementation of the gRPC strax server."""

from concurrent import futures
import numpy as np
import pandas as pd
import time
import fnmatch
import grpc
from . import straxrpc_pb2
from . import straxrpc_pb2_grpc
from .data_types import type_testers

_ONE_DAY_IN_SECONDS = 60 * 60 * 24



def fake_df(ncol=10,nrow=10):
    data = {}
    for c in range(ncol):
        data['col_{}'.format(c)] = np.random.random(nrow)
    return pd.DataFrame(data)

def empty_df(columns):
    data = {col:[] for col in columns}
    return pd.DataFrame(data)

def fake_arr(ncol=10,nrow=10):
    a = np.random.random((nrow,ncol))
    a.dtype=np.dtype([(('random column number {}'.format(c),'col_{}'.format(c)),np.float64) for c in range(ncol)])
    return a

def df_to_columns(df):
    cols = []
    for name in df.columns:
        dtype_name = str(df[name].dtype)
        for tester in type_testers:
            if tester.test(dtype_name):
                break
        else:
            continue

        info = straxrpc_pb2.ColumnInfo(
                  name=name,
                  dtype=tester.name,
            )
        try:
            values = tester.cast(df[name].values)
            data =  getattr(straxrpc_pb2,tester.column_class)(values=values)
            col_params = {'info': info, "index":df[name].index, tester.name:data}
            cols.append(straxrpc_pb2.DataColumn(**col_params))
        except:
            print("Could not transfer columns {}".format(name))
    return cols

def arr_to_columns(arr):
    cols = []
    for i, name in enumerate(arr.dtype.names):
        dtype_name = str(arr.dtype[i])
        for tester in type_testers:
            if tester.test(dtype_name):
                break
        else:
            continue

        info = straxrpc_pb2.ColumnInfo(
                  name=name,
                  dtype=tester.name,
            )
        values = arr[name].flatten()
        index = np.array(range(values.size), dtype=np.uint32)
        data =  getattr(straxrpc_pb2, tester.column_class)(values=values)
        col_params = {'info': info, "index":index, tester.name:data}
        cols.append(straxrpc_pb2.DataColumn(**col_params))
    return cols

def search_field(ctx, pattern):
    """
    Temporary fix, need to add pull request to have a flag to change this methods
    behavior so it returns a machine readable structure instead of printing to stdout.
    """
    match_list = []
    cache = dict()
    for d in ctx._plugin_class_registry:
        if d not in cache:
            cache.update(ctx._get_plugins((d,), run_id='0'))
        p = cache[d]

        for field_name in p.dtype.names:
            if fnmatch.fnmatch(field_name, pattern):
                match_list.append((field_name, d, p.__class__.__name__))
    return match_list

class StraxRPCServicer(straxrpc_pb2_grpc.StraxRPCServicer):

    def __init__(self, strax_context):
        print('Servicer started.')
        self.ctx = strax_context
        


    def SearchField(self, request, context):
        """
            TODO: Adapt search_field to accept flag for api usage of strax
            then change this method to:
                        match_list = self.ctx.search_field(pattern)
        """

        pattern = request.pattern
   
        match_list = search_field(self.ctx, pattern)
        for column, dataname, plugin in match_list:
            yield straxrpc_pb2.ColumnInfo(
                  name=column,
                  data_name=dataname,
                  plugin=plugin,
                 )


    def DataInfo(self, request, context):
      """

      """
      dataname = request.name
      df = self.ctx.data_info(dataname)
      for col in df_to_columns(df):
          yield col

    def GetDataframe(self, request, context):
        plugin_name = request.name
        run_id = request.run_id
        try:
            df = self.ctx.get_df(run_id, plugin_name) #
        except:
            columns = self.ctx.data_info(plugin_name)["Field name"].values
            df = empty_df(columns) #
        for col in df_to_columns(df):
            yield col

    def GetArray(self, request, context):
        plugin_name = request.name
        run_id = request.run_id
        try:
            arr = self.ctx.get_array(run_id, plugin_name) #
        except:
            arr = fake_arr() #
        for col in arr_to_columns(arr):
            yield col

    def SearchDataframeNames(self, request, context):
        pattern = request.pattern
        for d in self.ctx._plugin_class_registry:
            if fnmatch.fnmatch(d, pattern):
                yield straxrpc_pb2.PluginInfo(name=d,)

    def ShowConfig(self, request, context):
        name = request.name
        df = self.ctx.show_config(name)
        for col in df_to_columns(df):
            yield col

class StraxServer:
    def __init__(self, addr="localhost:50051", strax_context=None):
        self.addr = addr

    def serve(self, strax_context):
        server = grpc.server(futures.ThreadPoolExecutor(max_workers=10))
        straxrpc_pb2_grpc.add_StraxRPCServicer_to_server(
            StraxRPCServicer(strax_context), server)
        server.add_insecure_port(self.addr)
        server.start()
        try:
            while True:
                time.sleep(_ONE_DAY_IN_SECONDS)
        except KeyboardInterrupt:
            server.stop(0)

if __name__ == '__main__':
    from straxrpc import StraxServer
    server = StraxServer()
    ctx = strax.Context(
            storage=[],
            register_all=strax.xenon.plugins) 
    server.serve(ctx)



from setuptools import setup

setup(name='straxrpc',
      version='0.2.1',
      description='An RPC service for strax',
      url='http://github.com/jmosbacher/straxrpc',
      author='Yossi Mosbacher',
      author_email='joe.mosbacher@gmail.com',
      license='MIT',
      packages=['straxrpc'],
      install_requires=[
          'grpcio',
          'numpy',
          'pandas',
          'protobuf',
        #   ,
      ],
      extra_requires={
          "server": ['strax',
                    'fnmatch',]
      },
      zip_safe=False)